<?php

/*
Plugin Name: FahiPay Payment Gateway for WooCommerce
Plugin URI: https://wordpress.org/plugins/fahipay-payment-gateway-for-wc/
Description: FahiPay Payment gateway for woocommerce
Version: 1.0.0
Author: FahiPay
Author URI: https://fahipay.mv
*/


add_action('plugins_loaded', 'wc_fahipay_gateway_init', 0);

function wc_fahipay_gateway_init(){
    if(!class_exists('WC_Payment_Gateway')) return;
    class WC_fahipay_client extends WC_Payment_Gateway {
        public function __construct(){
            $this->id                 = 'wc_fahipay';
            $this->icon               = 'https://fahipay.mv/logo.png';
            $this->medthod_title      = 'FahiPay Client Integration';
            $this->has_fields         = false;
            $this->method_title       = "FahiPay";
            $this->method_description = "Payment via FahiPay Payment gateway";

            $this->init_form_fields();
            $this->init_settings();

            $this->title              = "FahiPay";
            $this->description        = "Pay securely by using your FahiPay account";
            $this->clientid           = $this->settings['clientid'];
            $this->clientsecret       = $this->settings['clientsecret'];
            
            $this->response_url	      = add_query_arg('wc-api', 'fahipay_request', home_url( '/' ));
            

            add_action( 'woocommerce_api_fahipay_request', array( $this, 'request' ) );
            add_action( 'woocommerce_api_fahipay_response', array( $this, 'response' ) );

            if (version_compare( WOOCOMMERCE_VERSION, '2.0.0', '>=' ))
            {
                add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( &$this, 'process_admin_options' ) );
            } 
            else 
            {
                add_action( 'woocommerce_update_options_payment_gateways', array( &$this, 'process_admin_options' ) );
            }
            
        }
        

        function init_form_fields()
        {
            $this->form_fields = array(
                'enabled' => array(
                'title' => __('Enable/Disable', 'client'),
                'type' => 'checkbox',
                'label' => __('Enable FahiPay Payment Module.', 'client'),
                'description' => __('This controls the description which the user sees during checkout.', 'client'),
                'default' => 'no'),

                'clientid' => array(
                'title' => __('Client ID', 'client'),
                'type' => 'text',
                'description' => __('Given to Merchant by FahiPay')),
                
                'clientsecret' => array(
                'title' => __('Client Secret', 'client'),
                'type' => 'text',
                'description' =>  __('Given to Merchant by FahiPay', 'client'),
                )
            );
        }

       public function admin_options()
       {
            echo '<h3>'.__('FahiPay Payment Gateway', 'client').'</h3>';
            echo '<p>'.__('FahiPay is a leading payment gateway for online shopping in Maldives').'</p>';
            echo '<table class="form-table">';
            $this->generate_settings_html();
            echo '</table>';
        }

         
        function payment_fields(){
            if($this->description) echo wpautop(wptexturize($this->description));
        }
        
        
        public function request(){ 
            global $woocommerce;
            if (isset($_GET['order_id']))
            {
                $order_id = wc_clean($_GET['order_id']);
            	$order    = new WC_Order($order_id);
            	if ($order->status=="pending")
            	{
                    $args = array(
                        'ShopID'         => $this->clientid,
                        'ShoppingCartID' => $order_id,
                        'TotalAmount'    => ($order->order_total*100),
                        'ReturnURL'      => home_url('/')."wc-api/fahipay_response/",
                        'ReturnErrorURL' => home_url('/')."wc-api/fahipay_response/",
                        'CancelURL'      => $order->get_cancel_order_url()
                    );
                    
                    $args['Signature']   = base64_encode(sha1($this->clientid.($this->clientsecret).$order_id.($this->clientsecret).($order->order_total*100).($this->clientsecret),true));
                    
                    echo '<!DOCTYPE html>
                            <html>
                            <head>
                            <style>
                            .loader {
                              border: 16px solid #f3f3f3;
                              border-radius: 50%;
                              border-top: 16px solid #3498db;
                              width: 120px;
                              height: 120px;
                              -webkit-animation: spin 2s linear infinite;
                              animation: spin 2s linear infinite;
                            }
                            
                            @-webkit-keyframes spin {
                              0% { -webkit-transform: rotate(0deg); }
                              100% { -webkit-transform: rotate(360deg); }
                            }
                            
                            @keyframes spin {
                              0% { transform: rotate(0deg); }
                              100% { transform: rotate(360deg); }
                            }
                            
                            input#submit_fahipay_payment_form {
                                display: none !important;
                            }
                            </style>
                            </head>
                            <body>
                            <form action="https://fahipay.mv/payment/" method="post" id="fahipay_payment_form">';
                            foreach($args as $key => $value)
                            {
                                echo "<input type='hidden' name='$key' value='$value'/>\n";
                            }
                            echo '<input style="display:none" type="submit" class="button-alt" id="submit_fahipay_payment_form" value="Submit"/>
                            </form>
                            <center style="margin-top:5%;">
                            <div class="loader"></div>
                            <p>Thank you for your order, redirecting...please wait.</p>
                            </center>
                            <script type="text/javascript">
                                document.getElementById("fahipay_payment_form").submit();
                            </script>
                            </body>
                            </html>';
            	}
            }
            die();
        }
        
        public function response(){ 
            global $woocommerce;
            $is_valid_response = false;
            
            if (isset($_GET['Success']) && isset($_GET['ShoppingCartID']))
            {
                $order_id         = wc_clean($_GET['ShoppingCartID']);
                $success          = wc_clean($_GET['Success']);
                
                if (intval($success)===1)
                {
                    if (isset($_GET['ApprovalCode']) && isset($_GET['Signature']))
                    {
                        $tid       = wc_clean($_GET['ApprovalCode']);  
                        if (preg_match('/^(FP[0-9]{14}[A-Z0-9]{4})$/', $tid, $matches))
                        {
                            $sig       = base64_encode(sha1($this->clientid.($this->clientsecret).$order_id.($this->clientsecret).$success.($this->clientsecret).$tid.($this->clientsecret), true));
                            
                            if ($sig==wc_clean($_GET['Signature']))
                            {
                                $is_valid_response = true;
                                $order = new WC_Order($order_id); 
                                $order->payment_complete();
                				$order->reduce_order_stock();
                				$order->add_order_note('FahiPay payment successful<br/>Fahipay transaction ID: '.$tid);
                				$woocommerce->cart->empty_cart();
                				wp_safe_redirect($this->get_return_url($order));
                            }
                        }
                    }
                } 
                else if (isset($_GET['ErrorMessage']))
                {
                    $is_valid_response = true;
                    wc_add_notice( __( 'Payment failed. Error: '.esc_html(wc_clean($_GET['ErrorMessage'])), 'gateway' ), 'error' );
    				wp_safe_redirect(wc_get_page_permalink('cart'));
                }
            }
            
            if (!$is_valid_response)
            {
                $woocommerce->cart->empty_cart();
                wc_add_notice( __( 'Payment failed. Security Error! Illegal access detected', 'gateway' ), 'error' );
                wp_safe_redirect(wc_get_page_permalink('cart'));
            }
            die();
        }
       
       
        function process_payment($order_id)
        {
    		return array(
            'result'    => 'success',
            'redirect'  => home_url('/')."wc-api/fahipay_request/?order_id=".$order_id
            );
        }
    
    }
   
    function woocommerce_add_fahipay_client_gateway($methods) {
        $methods[] = 'WC_fahipay_client';
        return $methods;
    }

    add_filter('woocommerce_payment_gateways', 'woocommerce_add_fahipay_client_gateway' );
}